package practice_tdd;

public class Money implements Expression {

	protected int amount;
	protected String currency;
	
	/**
	 * @param amount
	 * @param currency
	 */
	Money(int amount, String currency) {
		this.amount = amount;
		this.currency = currency;
	}

	/**
	 * @param obj
	 * @return
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		Money money = (Money)obj;
		return (amount == money.amount) && currency.equals(money.currency);
	}

	/**
	 * 달러 팩토리
	 * 
	 * @param amount
	 * @return
	 */
	static Money dollar(int amount) {
		return new Money(amount, "USD");
	}

	/**
	 * 프랑 팩토리
	 * 
	 * @param amount
	 * @return
	 */
	static Money franc(int amount) {
		return new Money(amount, "CHF");
	}

	/**
	 * 통화종류 반환
	 * 
	 * @return
	 */
	String currency() {
		return currency;
	}

	/**
	 * {@inheritDoc}
	 */
	public Money reduce(Bank bank, String to) {
		int rate = bank.getRate(currency, to);
		return new Money(amount / rate, to);
	}

	/**
	 * {@inheritDoc}
	 */
	public Expression plus(Expression addend) {
		return new Sum(this, addend);
	}

	/**
	* {@inheritDoc}
	*/
	public Expression times(int multiplier) {
		return new Money(amount * multiplier, currency);
	}

}